/* intercept time functions so that we can apply offset */

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <dlfcn.h>
#include <unistd.h>
#include <sys/time.h>

static time_t (*ds_old_time) (time_t *t);
static int (*ds_old_gettimeofday) (struct timeval *tv, struct timezone *tz);
static time_t ds_offset = 0;

static void
ds_initialize () __attribute__ ((constructor));

/* Run at startup */
static void
ds_initialize ()
{
    char *c;
    ds_old_time = dlsym(RTLD_NEXT, "time");
    if ( !ds_old_time ) {
        fprintf(stderr, "could not get time symbol from libc\n");
        exit(1);
    }

    ds_old_gettimeofday = dlsym(RTLD_NEXT, "gettimeofday");
    if ( !ds_old_gettimeofday ) {
        fprintf(stderr, "could not get gettimeofday symbol from libc\n");
        exit(1);
    }

    
    ds_offset = 0;
    c = getenv("DS_OFFSET");
    if (c) {
        ds_offset = atoi(c);
    }
}

time_t
time(time_t *t)
{
    time_t res;
    if (!ds_old_time) {
        fprintf(stderr, "dsintercept not initialised yet\n");
        exit(1);
    }
    
    (*ds_old_time)(&res);
    res += ds_offset;
    if (t) {
        *t = res;
    }
    return res;
}

int
gettimeofday(
    struct timeval *tv,
#ifdef GETTIMEOFDAY_HAS_TIMEZONE
    struct timezone *tz
#else
    void *tz
#endif
    )
{    
    int ret;

    if (!ds_old_gettimeofday) {
        fprintf(stderr, "dsintercept not initialised yet\n");
        exit(1);
    }

    ret = (*ds_old_gettimeofday)(tv, tz);
    
    if (tv) {
        tv->tv_sec += ds_offset;
    }
    return ret;
}
